from core.modules.vistrails_module import Module, ModuleError
from Scientific.IO import NetCDF
import numpy
import vtk

from SelfeBase import SelfeData

class Readers(object):
    namespace='IO'

class SelfeArrayReader(Readers, Module):
    name = 'Selfe Array Reader'

    def compute(self):
        fn = self.getInputFromPort("Filename")
        ar_name = self.getInputFromPort("Array Name")
        f = NetCDF.NetCDFFile(fn, 'r')
        ar = numpy.array(f.variables[ar_name])
        f.close()
        out = self.create_instance_of_type('edu.utah.sci.vistrails.numpyscipy', 'Numpy Array', 'numpy|array')
        out.set_array(ar)
        self.setResult("Output", out)

    @classmethod
    def register(cls,reg,basic):
        reg.add_module(cls, name=cls.name, namespace=cls.namespace)
        reg.add_input_port(cls, "Filename", (basic.String, 'Filename'))
        reg.add_input_port(cls, "Array Name", (basic.String, 'Array name'))
        reg.add_output_port(cls, "Output", (reg.get_module_by_name('edu.utah.sci.vistrails.numpyscipy', 'Numpy Array', 'numpy|array'), 'Output Array'))

class SelfeReader(Readers, Module):
    name = 'Selfe Grid Reader'
    
    def compute(self):
        fn = self.getInputFromPort("Filename")
        f = NetCDF.NetCDFFile(fn, 'r')
        x = numpy.array(f.variables['x'])
        y = numpy.array(f.variables['y'])
        plane_connectivity = numpy.array(f.variables['nm'])
        allz = numpy.array(f.variables['z'])

        zmod = self.forceGetInputFromPort("Z Scaling")
        if zmod == None:
            zmod = 1.

        ts = self.getInputFromPort("Timestep")
        if ts >= allz.shape[0]:
            ts = 0

        z = allz[ts].squeeze()
        vtkout = vtk.vtkUnstructuredGrid()
        
        # Just for memory sake, we should get rid of as much as we can...
        del allz
        
        i = 0
        vtk_pts = vtk.vtkPoints()
        for depth in range(z.shape[0]):
            print "depth " + str(depth) + " being populated"
            for node in range(x.shape[0]):
                ix = x[node]
                iy = y[node]
                iz = z[depth, node] * zmod
#                iz = float(depth* 40000)
                vtk_pts.InsertNextPoint(ix, iy, iz)
                i += 1
        
        print "Adding " + str(i) + " points to vtkUnstructuredGrid"
        vtkout.SetPoints(vtk_pts)
        print "Points added successfully..."
        
        # the points array is now populated...
        # let's do the connectivity now...each plane's
        # connectivity is defined in the plane_connectivity array
        num_tris = plane_connectivity.shape[1]
        n_depth = z.shape[0]
        n_slabs = n_depth - 1
        n_pts = x.shape[0]
        
        del x
        del y
        del z

        for i in range(n_slabs):
            top_offset = i * n_pts
            bot_offset = (i+1) * n_pts
            print "Forming prisms for slab " + str(i)
            for t in range(num_tris):
                tri = plane_connectivity[:,t].squeeze()
                lst = vtk.vtkIdList()
                lst.InsertNextId(tri[0]-1+top_offset)
                lst.InsertNextId(tri[1]-1+top_offset)
                lst.InsertNextId(tri[2]-1+top_offset)
                lst.InsertNextId(tri[0]-1+bot_offset)
                lst.InsertNextId(tri[1]-1+bot_offset)
                lst.InsertNextId(tri[2]-1+bot_offset)
                vtkout.InsertNextCell(13, lst)


        scalars = self.forceGetInputFromPort("Scalars")
        if scalars != None:
            try:
                s = numpy.array(f.variables[scalars])
            except:
                raise ModuleError("Could not find array named:  " + scalars)

            s = s[ts].squeeze().flatten()
            s_ar = vtk.vtkFloatArray()
            s_ar.SetName(scalars)
            for i in range(s.shape[0]):
                s_ar.InsertNextValue(s[i])

            s_ar.SetName(scalars)
            vtkout.GetPointData().SetScalars(s_ar)
            
        
        out = self.create_instance_of_type('edu.utah.sci.vistrails.vtk', 'vtkUnstructuredGrid')
        out.vtkInstance = vtkout
        self.setResult("Output Mesh", out)

    @classmethod
    def register(cls, reg, basic):
        reg.add_module(cls, namespace=cls.namespace, name=cls.name)
        reg.add_input_port(cls, "Filename", (basic.String, 'Filename'))
        reg.add_input_port(cls, "Timestep", (basic.Integer, 'Timestep'))
        reg.add_input_port(cls, "Z Scaling", (basic.Float, 'Z scale factor'))
        reg.add_input_port(cls, "Scalars", (basic.String, 'Scalar field'))
        reg.add_output_port(cls, "Output Mesh", (reg.get_module_by_name('edu.utah.sci.vistrails.vtk','vtkUnstructuredGrid'), 'Output Mesh'))

class GridPakReader(Readers, Module):
    name = 'Gridpak Reader'
    def compute(self):
        fn = self.getInputFromPort("Filename")
        f = NetCDF.NetCDFFile(fn, 'r')

        grid_x = 'x_rho'
        grid_y = 'y_rho'
        grid_z = 'z_rho'

        grid_name = self.forceGetInputFromPort("Grid Name")
        if grid_name:
            grid_x = 'x_' + grid_name
            grid_y = 'y_' + grid_name
            grid_z = 'z_' + grid_name
        
        x = numpy.array(f.variables[grid_x])
        y = numpy.array(f.variables[grid_y])
        z = numpy.array(f.variables[grid_z])
        h = numpy.array(f.variables['h'])

        f.close()

        zmod = self.forceGetInputFromPort("Z Scaling")
        if zmod == None:
            zmod = 1.

        vtkout = self.create_instance_of_type('edu.utah.sci.vistrails.vtk', 'vtkStructuredGrid')
        vtkout.vtkInstance = vtk.vtkStructuredGrid()
        pts = vtk.vtkPoints()

        for k in range(z.shape[2]):
            for j in range(z.shape[1]):
                for i in range(z.shape[0]):
                    x_coord = x[i,j]
                    y_coord = y[i,j]
#                    z_coord = h[i,j] / z.shape[2] * float(k)
                    z_coord = z[i,j,k] * float(zmod)

                    pts.InsertNextPoint(x_coord, y_coord, z_coord * zmod)

        print z.shape
        vtkout.vtkInstance.SetDimensions(z.shape)
        
        vtkout.vtkInstance.SetPoints(pts)
        self.setResult("Output Mesh", vtkout)

    @classmethod
    def register(cls, reg, basic):
        reg.add_module(cls, namespace=cls.namespace, name=cls.name)
        reg.add_input_port(cls, "Filename", (basic.String, 'Filename'))
        reg.add_input_port(cls, "Z Scaling", (basic.Float, 'Z scale factor'))
        reg.add_input_port(cls, "Grid Name", (basic.String, 'Grid name'), True)
        reg.add_output_port(cls, "Output Mesh", (reg.get_module_by_name('edu.utah.sci.vistrails.vtk','vtkStructuredGrid'), 'Output Mesh'))
        
